#include <cmath>
#include "raylib.h"
#include "raymath.h"

#define MAX_DEPTH 10

Vector2 draw_line(Vector2 start, float angleDeg, float lenghth)
{
  angleDeg += 180.0f;
  float angle = (angleDeg * PI) / 180.0f;
  float nx = lenghth * std::sin(angle);
  float ny = lenghth * std::cos(angle);
  Vector2 end = {start.x + nx, start.y + ny};
  DrawLineEx(start, end, 2.0f, PURPLE);
  return end;
}

void draw_branch(Vector2 start, float angle, float len, int dep)
{
  Vector2 next = draw_line(start, angle, len);

  if (len < MAX_DEPTH)
    return;

  float next_len = 0.7f;
  draw_branch(next, angle + 45, len * next_len, dep + 1);
  draw_branch(next, angle - 45, len * next_len, dep + 1);
}

int main(void)
{
  const int screenWidth = 800;
  const int screenHeight = 800;

  InitWindow(screenWidth, screenHeight, "raylib");

  Camera2D camera = {0};
  camera.target = (Vector2){0.0f, 0.0f};
  camera.offset = (Vector2){screenWidth / 2, screenHeight};
  camera.rotation = 0.0f;
  camera.zoom = 1.0f;

  SetTargetFPS(60);
  Vector2 start = {0};
  while (!WindowShouldClose())
  {

    BeginDrawing();
    ClearBackground(RAYWHITE);
    BeginMode2D(camera);

    DrawCircle(0, 0, 10.0f, RED);
    draw_branch(start, 0, screenHeight / 4, 0);

    EndMode2D();
    EndDrawing();
  }

  CloseWindow();
  return 0;
}